define([
    "jquery", "underscore", "backbone",
    "views/snippet-translate"
], function(
    $, _, Backbone,
    SnippetView
    ) {
    return SnippetView.extend({
        events: {
            "click": "preventPropagation", //stops checkbox / radio reacting.
            "mousedown": "mouseDownHandler",
            "mouseup": "mouseUpHandler"
        },
        mouseDownHandler: function(mouseDownEvent) {
            mouseDownEvent.stopPropagation();
            mouseDownEvent.preventDefault();
            var that = this;
            //popover
            $(".popover").remove();
            this.$el.popover("show");
            $("#accordionValidations").find(".order-arrow").hide();
            $(".popover #save").on("click", this.saveHandler(that));
            $(".popover #cancel").on("click", this.cancelHandler(that));
        },
        preventPropagation: function(e) {
            e.stopPropagation();
            e.preventDefault();
        },
        mouseUpHandler: function(mouseUpEvent) {
            $("body").off("mousemove");
        },
        saveHandler: function(boundContext) {
            function saveFields(fields) {
                _.each(fields, function (e) {
                    var $e = $(e),
                        type = $e.attr("data-type"),
                        name = $e.attr("id");
                    switch (type) {
                        case "checkbox":
                            boundContext.model.setField(name, $e.is(":checked"));
                            break;
                        case "input":
                            boundContext.model.setField(name, $e.val());
                            break;
                        case "hidden":
                            boundContext.model.setField(name, $e.val());
                            break;
                        case "textarea":
                            boundContext.model.setField(name, $e.val());
                            break;
                        case "textarea-split":
                            boundContext.model.setField(name,
                                _.chain($e.val().split("\n"))
                                    .map(function (t) {
                                        return $.trim(t)
                                    })
                                    .filter(function (t) {
                                        return t.length > 0
                                    })
                                    .value()
                            );
                            break;
                        case "select":
                            var valarr = _.map($e.find("option"), function (e) {
                                return {
                                    value: e.value,
                                    selected: e.selected,
                                    label: $(e).text()
                                };
                            });
                            boundContext.model.setField(name, valarr);
                            break;
                        case "key-value":
                            keyValues = [];
                            _.each($e.find( ".keyField" ),function(keyInput){
                                var $key = $(keyInput);
                                var valueId = $key.attr("id").replace("key","value");
                                var $value = $("#"+valueId);
                                keyValues.push({"key":$key.val(),"value":$value.val()});
                            });
                            boundContext.model.setField(name, keyValues);
                            break;
                    }
                });
            }

            function saveValidations(fields) {
                _.each(fields, function (e) {
                    var $e = $(e),
                        type = $e.attr("data-type"),
                        fieldID = $e.attr("id");

                    if (fieldID.split("_")[1] != "activator") {
                        switch (type) {
                            case "checkbox":
                                boundContext.model.setValidation(fieldID, $e.is(":checked"));
                                break;
                            case "input":
                                boundContext.model.setValidation(fieldID, $e.val());
                                break;
                            case "hidden":
                                boundContext.model.setValidation(fieldID, $e.val());
                                break;
                            case "textarea":
                                boundContext.model.setValidation(fieldID, $e.val());
                                break;
                            case "textarea-split":
                                boundContext.model.setValidation(fieldID,
                                    _.chain($e.val().split("\n"))
                                        .map(function (t) {
                                            return $.trim(t)
                                        })
                                        .filter(function (t) {
                                            return t.length > 0
                                        })
                                        .value()
                                );
                                break;
                            case "select":
                                var valarr = _.map($e.find("option"), function (e) {
                                    return {
                                        value: e.value,
                                        selected: e.selected,
                                        label: $(e).text()
                                    };
                                });
                                boundContext.model.setValidation(fieldID, valarr);
                                break;
                        }
                    }
                });
            }

            return function(mouseEvent) {
                blockInfoPopupOnSectionChange = false;
                mouseEvent.preventDefault();
                var fields = $("#structure ").find(".field");
                saveFields(fields);
                fields = $("#validation").find(".field");
                saveValidations(fields);
                boundContext.model.trigger("change");
                $(".popover").remove();
            }
        },
        cancelHandler: function(boundContext) {
            return function(mouseEvent) {
                mouseEvent.preventDefault();
                $(".popover").remove();
                boundContext.model.trigger("change");
            }
        }
    });
});