define([
    "jquery", "underscore", "backbone"
], function ($, _, Backbone) {
    return Backbone.View.extend({
        tagName: "div",
        className: "formFactoryComponent",
        initialize: function () {
            this.template = _.template(_snippetTemplates[this.model.idNodeType()]);
            this.popoverTemplates = {};
            var that = this;
            _.each(_popoverTemplates,function(popover, key, list) {
                that.popoverTemplates[key] = _.template(popover);
            })
        },
        render: function (withAttributes) {
            var that = this;
            // Delete untranslatable fields
            var translatableFields = this.removeUntranslatableFields(that, JSON.parse(JSON.stringify(that.model.get("fields"))));
            // Put each validation field from the snippet model to validations object
            var validations;
            if (!_.isUndefined(that.model.get("validations"))) {
                validations = this.mergeValidations(that.model.get("validations"), JSON.parse(JSON.stringify(_validationsData)));
            }
            // Delete not activate validations for this snippet
            validations = this.activatedValidations(validations);
            // Delete not editabe Validation fields
            validations = this.removeUntranslatableValidationFields(validations);

            var content = _.template(_popoverTemplates.main)({
                "title": that.model.get("title"),
                "items": translatableFields,
                "validations": validations,
                "mode":'translate',
                "popoverTemplates": that.popoverTemplates
            });
            var values = that.model.getValues();
            values["validations"] = that.model.getValidations();
            if (withAttributes && (_.size(translatableFields) || _.size(validations))) {
                return this.$el.html(
                    that.template(values)
                ).attr({
                        "data-content": content,
                        "data-title": that.model.get("title"),
                        "data-trigger": "manual",
                        "data-html": true,
                        "data-placement": "right"
                    });
            } else {
                return this.$el.html(
                    that.template(values)
                )
            }
        },
        mergeValidations: function(modelValidations, validations) {
            for (var validationName in modelValidations) {
                for (var validationFieldName in modelValidations[validationName]["fields"]) {
                    validations[validationName]["fields"][validationFieldName]["value"] = modelValidations[validationName]["fields"][validationFieldName]["value"];
                    validations[validationName]["fields"][validationFieldName]["validationJcrId"] = modelValidations[validationName]["fields"][validationFieldName]["validationJcrId"];
                }
            }
            return validations;
        },
        activatedValidations: function(validations) {
            var validationToRemove = [];
            _.each(validations, function(validation, nodename) {
                if (!validation["fields"]["activator"]["value"]) {
                    validationToRemove.push(nodename);
                }
            });
            if (_.size(validationToRemove) >= 1) {
                _.each(validationToRemove, function(val) {
                    delete validations[val];
                });
            }
            return validations;
        },
        removeUntranslatableFields: function (snippet, translatableFields) {
            _.each(snippet.model.get("fields"), function(value, key) {
                if (!value.translatable) {
                    delete translatableFields[key];
                }
            });
            return translatableFields;
        },
        removeUntranslatableValidationFields: function (validations) {
            _.each(validations, function(validation) {
                _.each(validation.fields, function (field, k) {
                    if (!field.translatable) {
                        field.type = "hidden";
                    }
                })
            });
            return validations;
        }
    });
});